import React, { useEffect, lazy, Suspense, useState } from 'react';
import { BrowserRouter, Routes, Route, Navigate, useLocation } from 'react-router-dom';
import { Layout } from './components/Layout';
import { useThemeStore } from './store/theme';
import HomePage from './pages/HomePage';
import { Toaster } from 'react-hot-toast';
import { GroupChatRouteGuard } from './components/GroupChatRouteGuard';
import { setupMutedUsersListener } from './hooks/useMutedUsers';
import { listenerManager } from './lib/listenerManager';
import { useFriendsStore } from './store/friends';
import { useSocketConnection } from './hooks/useSocketConnection';
import { useAuthStore } from './store/auth';
import { isValidUserId } from './utils/idOnlyIdentification';
import { initProfileOptimizations } from './lib/profileOptimizations';
import { LeaderboardPage } from './pages/LeaderboardPage';
import { AccountConversionModalManager } from './components/AccountConversionModalManager';
import { clearAllStreakCaches } from './features/streaks/hooks/useStreak';
import { trackPageView } from './lib/analytics';

// Add TypeScript type declaration for window.socket
declare global {
  interface Window {
    socket?: any;
    // Add validation helper for ID-only approach
    __validateUserId?: (id: string) => boolean;
  }
}

// Fix Socket URL global initialization to prevent "process is not defined" errors
const initializeSocketUrl = () => {
  try {
    const protocol = window.location.protocol;
    const hostname = window.location.hostname;
    const socketUrl = `${protocol}//${hostname}:3000`;
    
    // Set as global property
    (window as any).SOCKET_URL = socketUrl;
    console.log(`[App] Initialized Socket URL: ${socketUrl}`);
  } catch (e) {
    console.error('[App] Error initializing Socket URL:', e);
    // Fallback to localhost if needed
    (window as any).SOCKET_URL = 'http://localhost:3000';
  }
};

// Initialize at the app level
if (typeof window !== 'undefined') {
  initializeSocketUrl();
  
  // Add a global validation helper for other parts of the app
  window.__validateUserId = (id: string) => {
    const isValid = isValidUserId(id);
    if (!isValid && process.env.NODE_ENV === 'development') {
      console.warn(`[ID-Only] Invalid user ID format detected: ${id}`);
    }
    return isValid;
  };
}

// Fallback component for pages that might not exist
const FallbackPage = () => (
  <div className="min-h-screen flex flex-col items-center justify-center bg-gray-50 dark:bg-gray-900">
    <h1 className="text-2xl font-bold mb-4">Page Not Available</h1>
    <p className="mb-4 text-center max-w-md">This page is not available in the current version.</p>
    <a href="/" className="px-4 py-2 bg-blue-500 text-white rounded-md hover:bg-blue-600 transition-colors">
      Return Home
    </a>
  </div>
);

// Create direct component definitions for pages that don't exist
const LoginPage = () => (
  <div className="min-h-screen flex flex-col items-center justify-center bg-gray-50 dark:bg-gray-900">
    <h1 className="text-2xl font-bold mb-4">Login</h1>
    <p className="mb-4 text-center max-w-md">This page is not available in the current version.</p>
    <a href="/" className="px-4 py-2 bg-blue-500 text-white rounded-md hover:bg-blue-600 transition-colors">
      Return Home
    </a>
  </div>
);

const SignupPage = () => (
  <div className="min-h-screen flex flex-col items-center justify-center bg-gray-50 dark:bg-gray-900">
    <h1 className="text-2xl font-bold mb-4">Sign Up</h1>
    <p className="mb-4 text-center max-w-md">This page is not available in the current version.</p>
    <a href="/" className="px-4 py-2 bg-blue-500 text-white rounded-md hover:bg-blue-600 transition-colors">
      Return Home
    </a>
  </div>
);

const ForgotPasswordPage = () => (
  <div className="min-h-screen flex flex-col items-center justify-center bg-gray-50 dark:bg-gray-900">
    <h1 className="text-2xl font-bold mb-4">Forgot Password</h1>
    <p className="mb-4 text-center max-w-md">This page is not available in the current version.</p>
    <a href="/" className="px-4 py-2 bg-blue-500 text-white rounded-md hover:bg-blue-600 transition-colors">
      Return Home
    </a>
  </div>
);

const VerifyEmailPage = () => (
  <div className="min-h-screen flex flex-col items-center justify-center bg-gray-50 dark:bg-gray-900">
    <h1 className="text-2xl font-bold mb-4">Verify Email</h1>
    <p className="mb-4 text-center max-w-md">This page is not available in the current version.</p>
    <a href="/" className="px-4 py-2 bg-blue-500 text-white rounded-md hover:bg-blue-600 transition-colors">
      Return Home
    </a>
  </div>
);

const ResetPasswordPage = () => (
  <div className="min-h-screen flex flex-col items-center justify-center bg-gray-50 dark:bg-gray-900">
    <h1 className="text-2xl font-bold mb-4">Reset Password</h1>
    <p className="mb-4 text-center max-w-md">This page is not available in the current version.</p>
    <a href="/" className="px-4 py-2 bg-blue-500 text-white rounded-md hover:bg-blue-600 transition-colors">
      Return Home
    </a>
  </div>
);

// Lazy load existing pages
const ChatPage = lazy(() => import('./pages/ChatPage').then(module => ({ default: module.ChatPage })));
const AboutPage = lazy(() => import('./pages/AboutPage').then(module => ({ default: module.AboutPage })));
const GuidelinesPage = lazy(() => import('./pages/GuidelinesPage').then(module => ({ default: module.GuidelinesPage })));
const SettingsPage = lazy(() => import('./pages/SettingsPage').then(module => ({ default: module.SettingsPage })));
const HelpPage = lazy(() => import('./pages/HelpPage').then(module => ({ default: module.HelpPage })));
const TermsPage = lazy(() => import('./pages/TermsPage').then(module => ({ default: module.TermsPage })));
const PrivacyPage = lazy(() => import('./pages/PrivacyPage').then(module => ({ default: module.PrivacyPage })));
const CookiePage = lazy(() => import('./pages/CookiePage').then(module => ({ default: module.CookiePage })));
const ContactPage = lazy(() => import('./pages/ContactPage').then(module => ({ default: module.ContactPage })));
const DebugPage = lazy(() => import('./pages/Debug'));

// Group chat pages
const GroupChatLayout = lazy(() => import('./features/group-chat/components/GroupChatLayout').then(module => ({ default: module.GroupChatLayout })));
const GroupChatMainPage = lazy(() => import('./features/group-chat/pages/GroupChatMainPage'));
const FriendsPage = lazy(() => import('./features/group-chat/pages/FriendsPage').then(module => ({ default: module.FriendsPage })));
const DirectMessagePage = lazy(() => import('./features/group-chat/pages/DirectMessagePage').then(module => ({ default: module.DirectMessagePage })));
const AdminPage = lazy(() => import('./features/group-chat/pages/AdminPage').then(module => ({ default: module.AdminPage })));
const DashboardPage = lazy(() => import('./features/group-chat/pages/DashboardPage').then(module => ({ default: module.DashboardPage })));

// Landing Pages
const RandomGirlsChat = lazy(() => import('./pages/landing/RandomGirlsChat'));
const DesiGlobalChat = lazy(() => import('./pages/landing/DesiGlobalChat'));
const AnonymousGirlChat = lazy(() => import('./pages/landing/AnonymousGirlChat'));
const GlobalChatRooms = lazy(() => import('./pages/landing/GlobalChatRooms'));
const RandomGirlsInstant = lazy(() => import('./pages/landing/RandomGirlsInstant'));
const ForeignGirlsChat = lazy(() => import('./pages/landing/ForeignGirlsChat'));
const LateNightChat = lazy(() => import('./pages/landing/LateNightChat'));
const MeetGirlsOnline = lazy(() => import('./pages/landing/MeetGirlsOnline'));
const CollegeGirlsChat = lazy(() => import('./pages/landing/CollegeGirlsChat'));
const NearbyGirlsChat = lazy(() => import('./pages/landing/NearbyGirlsChat'));

// Add Confession Wall to the existing group chat routes
const ConfessionWall = lazy(() => import('./pages/ConfessionWall'));

// Route wrapper that validates user ID parameters
const ValidatedIdRoute = ({ 
  element, 
  paramName = 'partnerId',
  ...props 
}: { 
  element: React.ReactNode; 
  paramName?: string;
}) => {
  useEffect(() => {
    const path = window.location.pathname;
    const parts = path.split('/');
    const potentialId = parts[parts.length - 1];
    
    if (potentialId && !isValidUserId(potentialId) && process.env.NODE_ENV === 'development') {
      console.warn(`[ID-Only] Route parameter "${paramName}" does not appear to be a valid user ID: ${potentialId}`);
    } else if (potentialId && isValidUserId(potentialId) && process.env.NODE_ENV === 'development') {
      console.info(`[ID-Only] Valid user ID detected in route: ${potentialId}`);
    }
  }, [paramName]);
  
  return <>{element}</>;
};

const PageLoader = () => (
  <div className="min-h-screen flex items-center justify-center bg-gray-50 dark:bg-gray-900">
    <div className="animate-spin rounded-full h-12 w-12 border-t-2 border-b-2 border-blue-500"></div>
  </div>
);

// Analytics tracker component
const RouteTracker = () => {
  const location = useLocation();
  
  useEffect(() => {
    // Track page view on location change
    trackPageView(location.pathname);
  }, [location]);
  
  return null;
};

function App() {
  const { isDark } = useThemeStore();
  const { cleanupListeners: cleanupFriendsListeners } = useFriendsStore();
  const { user } = useAuthStore();
  const [isLoading, setIsLoading] = useState(true);
  
  // Initialize socket connection - this handles auth state automatically
  useSocketConnection();

  // Initialize profile optimizations
  useEffect(() => {
    // Initialize profile optimizations to reduce database reads
    const { stopTelemetry, logStats } = initProfileOptimizations({
      enableTelemetry: process.env.NODE_ENV === 'development',
      telemetryIntervalMs: 120000, // Log every 2 minutes in development
      logInitialStats: false
    });
    
    // Add a debug helper to window in development mode
    if (process.env.NODE_ENV === 'development') {
      (window as any).__profileStats = logStats;
    }
    
    // Clean up on unmount
    return () => {
      stopTelemetry();
    };
  }, []);

  // Check if user auth is ready
  useEffect(() => {
    // Short timeout to ensure auth state is settled
    const timer = setTimeout(() => {
      setIsLoading(false);
    }, 500);
    
    return () => clearTimeout(timer);
  }, [user]);

  // Log ID-based telemetry in development mode
  useEffect(() => {
    if (process.env.NODE_ENV === 'development' && user?.id) {
      console.info(`[ID-Only] App initialized with authenticated user ID: ${user.id}`);
    }
  }, [user?.id]);

  // Initialize centralized listeners when app loads
  useEffect(() => {
    if (typeof window !== 'undefined') {
      console.log('[App] Initializing centralized listeners');
      
      // Setup listener for muted users using the centralized manager
      setupMutedUsersListener();
      
      // Periodically optimize listeners to reduce Firestore reads
      const optimizationInterval = setInterval(() => {
        listenerManager.optimizeListeners();
      }, 5 * 60 * 1000); // Every 5 minutes
      
      // Clean up on app unmount
      return () => {
        clearInterval(optimizationInterval);
        
        // Clean up friends store listeners
        console.log('[App] Cleaning up friends store listeners');
        cleanupFriendsListeners();
      };
    }
  }, [cleanupFriendsListeners]);

  useEffect(() => {
    document.documentElement.classList.toggle('dark', isDark);
    document.body.className = isDark ? 'dark bg-gray-900' : 'bg-gray-50';

    const mediaQuery = window.matchMedia('(prefers-color-scheme: dark)');
    const handleChange = (e: MediaQueryListEvent) => {
      if (!isDark) {
        document.documentElement.classList.toggle('dark', e.matches);
        document.body.className = e.matches ? 'dark bg-gray-900' : 'bg-gray-50';
      }
    };

    mediaQuery.addEventListener('change', handleChange);
    return () => mediaQuery.removeEventListener('change', handleChange);
  }, [isDark]);

  // Initialize online status service
  useEffect(() => {
    if (typeof window !== 'undefined') {
      console.log('[App] Initializing online status service');
      
      // Import dynamically to avoid initialization issues
      import('./services/onlineStatusService').then(module => {
        const service = module.onlineStatusService;
        
        try {
          // Use the socket from the import directly instead of window.socket
          import('./lib/socket').then(socketModule => {
            const { socket } = socketModule;
            
            if (socket.connected) {
              service.initialize();
            } else {
              // Set up one-time connect listener
              const connectHandler = () => {
                service.initialize();
                socket.off('connect', connectHandler);
              };
              
              socket.on('connect', connectHandler);
              console.log('[App] Waiting for socket connection before initializing online status service');
            }
          });
        } catch (error) {
          console.error('[App] Error initializing online status service:', error);
        }
      });
      
      return () => {
        // Clean up on unmount
        import('./services/onlineStatusService').then(module => {
          module.onlineStatusService.cleanup();
        });
      };
    }
  }, []);

  // Clear streak caches when the app starts
  useEffect(() => {
    clearAllStreakCaches();
  }, []);

  if (isLoading) {
    return <div className="flex items-center justify-center h-screen">Loading...</div>;
  }

  return (
    <>
      <BrowserRouter>
        <RouteTracker />
        <Routes>
          <Route path="/" element={<Layout />}>
            <Route index element={<HomePage />} />
            <Route path="chat" element={
              <Suspense fallback={<PageLoader />}>
                <ChatPage />
              </Suspense>
            } />
            
            {/* Regular pages */}
            <Route path="about" element={
              <Suspense fallback={<PageLoader />}>
                <AboutPage />
              </Suspense>
            } />
            <Route path="guidelines" element={
              <Suspense fallback={<PageLoader />}>
                <GuidelinesPage />
              </Suspense>
            } />
            <Route path="settings" element={
              <Suspense fallback={<PageLoader />}>
                <SettingsPage />
              </Suspense>
            } />
            <Route path="help" element={
              <Suspense fallback={<PageLoader />}>
                <HelpPage />
              </Suspense>
            } />
            <Route path="terms" element={
              <Suspense fallback={<PageLoader />}>
                <TermsPage />
              </Suspense>
            } />
            <Route path="privacy" element={
              <Suspense fallback={<PageLoader />}>
                <PrivacyPage />
              </Suspense>
            } />
            <Route path="cookies" element={
              <Suspense fallback={<PageLoader />}>
                <CookiePage />
              </Suspense>
            } />
            <Route path="contact" element={
              <Suspense fallback={<PageLoader />}>
                <ContactPage />
              </Suspense>
            } />
            <Route path="debug" element={
              <Suspense fallback={<PageLoader />}>
                <DebugPage />
              </Suspense>
            } />
            
            {/* Landing Pages */}
            <Route path="talk-to-random-girls" element={
              <Suspense fallback={<PageLoader />}>
                <RandomGirlsChat />
              </Suspense>
            } />
            <Route path="desi-global-girls-chat" element={
              <Suspense fallback={<PageLoader />}>
                <DesiGlobalChat />
              </Suspense>
            } />
            <Route path="anonymous-girl-chat" element={
              <Suspense fallback={<PageLoader />}>
                <AnonymousGirlChat />
              </Suspense>
            } />
            <Route path="global-girls-chat-room" element={
              <Suspense fallback={<PageLoader />}>
                <GlobalChatRooms />
              </Suspense>
            } />
            <Route path="chat-with-girls-online" element={
              <Suspense fallback={<PageLoader />}>
                <RandomGirlsInstant />
              </Suspense>
            } />
            <Route path="foreign-girls-chat" element={
              <Suspense fallback={<PageLoader />}>
                <ForeignGirlsChat />
              </Suspense>
            } />
            <Route path="late-night-girl-chat" element={
              <Suspense fallback={<PageLoader />}>
                <LateNightChat />
              </Suspense>
            } />
            <Route path="meet-girls-online-chat" element={
              <Suspense fallback={<PageLoader />}>
                <MeetGirlsOnline />
              </Suspense>
            } />
            <Route path="college-girls-random-chat" element={
              <Suspense fallback={<PageLoader />}>
                <CollegeGirlsChat />
              </Suspense>
            } />
            <Route path="chat-with-girls-near-or-far" element={
              <Suspense fallback={<PageLoader />}>
                <NearbyGirlsChat />
              </Suspense>
            } />
            <Route path="leaderboard" element={
              <Suspense fallback={<PageLoader />}>
                <LeaderboardPage />
              </Suspense>
            } />
          </Route>
            
          {/* Group Chat with nested routes */}
          <Route path="group-chat" element={
            <Suspense fallback={<PageLoader />}>
              <GroupChatRouteGuard>
                <GroupChatLayout />
              </GroupChatRouteGuard>
            </Suspense>
          }>
            <Route index element={
              <Suspense fallback={<PageLoader />}>
                <GroupChatMainPage />
              </Suspense>
            } />
            <Route path="friends" element={
              <Suspense fallback={<PageLoader />}>
                <FriendsPage />
              </Suspense>
            } />
            <Route path="dm" element={
              <Suspense fallback={<PageLoader />}>
                <DirectMessagePage />
              </Suspense>
            } />
            <Route path="dm/:partnerId" element={
              <Suspense fallback={<PageLoader />}>
                <ValidatedIdRoute 
                  element={<DirectMessagePage />} 
                  paramName="partnerId" 
                />
              </Suspense>
            } />
            <Route path="admin" element={
              <Suspense fallback={<PageLoader />}>
                <AdminPage />
              </Suspense>
            } />
            <Route path="confession-wall" element={
              <Suspense fallback={<PageLoader />}>
                <ConfessionWall />
              </Suspense>
            } />
            <Route path="dashboard" element={
              <Suspense fallback={<PageLoader />}>
                <DashboardPage />
              </Suspense>
            } />
          </Route>
        </Routes>
        <Toaster 
          position="top-center"
          toastOptions={{
            duration: 3000,
            style: {
              background: '#1B2B4A',
              color: '#fff',
              borderRadius: '8px',
            },
            success: {
              iconTheme: {
                primary: '#4ade80',
                secondary: '#1B2B4A',
              },
            },
            error: {
              iconTheme: {
                primary: '#ef4444',
                secondary: '#1B2B4A',
              },
            },
          }}
        />
        <AccountConversionModalManager />
      </BrowserRouter>
    </>
  );
}

export default App;